from __future__ import annotations

from PySide6.QtWidgets import QWidget
from PySide6.QtGui import QImage, QMouseEvent
from PySide6.QtCore import Signal

from tat.image_entry import ImageEntry
from tat.layer_data import LayerData

from typing import Final, List


class ClusterImageEntry(ImageEntry):
    """
    Extends ImageEntry. Add a list of LayerData that contains all the cluster layers LayerData.
    """

    double_clicked = Signal(QWidget)

    def __init__(self, parent: QWidget, image: QImage, name: str, image_path: str, array_path: str,
                 layers_data: List[LayerData]):
        """
        Instantiate a ClusterImageEntry object.

        :param parent: The widget calling the method.
        :param image: The image that will be used to draw the preview thumbnail.
        :param image_path: The path of the cluster image.
        :param name: The name that will be showed below the thumbnail.
        :param layers_data: A list of LayerData containing all the information about a layer.
        """
        super(ClusterImageEntry, self).__init__(parent, image, name, image_path, array_path)
        self.layers_data: Final[List[LayerData]] = layers_data

    def mouseDoubleClickEvent(self, event: QMouseEvent) -> None:
        self.double_clicked.emit(self)

    def layer_count(self) -> int:
        return len(self.layers_data)

    def add_layer_data(self, layer_data: LayerData) -> None:
        self.layers_data.append(layer_data)

    def get_layer_data(self, index: int) -> LayerData:
        return self.layers_data[index]

    def remove_layer_data(self, index: int) -> LayerData:
        return self.layers_data.pop(index)



import os
from pathlib import Path
from typing import Optional, List, Tuple

import numpy as np
import cv2 as cv

from PySide6.QtWidgets import QFileDialog, QLabel, QLayout
from PySide6.QtCore import QObject, Slot, Signal, QThreadPool, QRunnable

from tat.clustering import Tat
from tat.checkable_image_entry import CheckableImageEntry
from tat.preview_window import PreviewWindow
from tat.cluster_image_entry import ClusterImageEntry
from tat.cluster_editor import ClusterEditor
from tat.layer_data import LayerData
from tat.progress_window import ProgressWindow
from tat.ui_main_window import Ui_MainWindow
from tat.utils import load_image, apply_colormap, create_cluster, array3d_to_pixmap


class ClusteringSignals(QObject):
    finished = Signal()
    progress = Signal(int, tuple)


class ClusteringWorker(QRunnable):
    """
    Runner that will be ran while the progress bar is showing.
    """

    signals = ClusteringSignals()
    is_running = False

    def __init__(self, entries: List[CheckableImageEntry], cluster_count: int, run_count: int, max_iter_count: int,
                 output_directory: str):
        super(ClusteringWorker, self).__init__()
        self.entries = entries
        self.cluster_count = cluster_count
        self.run_count = run_count
        self.max_iter_count = max_iter_count
        self.output_directory = output_directory

    @Slot()
    def run(self) -> None:
        """
        Performs the clustering computing.
        """
        self.is_running = True
        for index, ime in enumerate(self.entries):
            if not self.is_running:
                break

            input_basename_no_ext = (lambda basename: basename[0:basename.rfind(".")])(os.path.basename(ime.image_path))
            layers, cluster = Tat.generate_layers(np.asarray(cv.imread(ime.image_path, flags=cv.IMREAD_GRAYSCALE)),
                                                  self.cluster_count, self.run_count, self.max_iter_count)

            layers_data: List[LayerData] = []
            for i, layer in enumerate(layers):
                output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_layer_{i}")
                output_image_path = f"{output_path_no_ext}.png"
                output_array_path = f"{output_path_no_ext}.npy"
                np.save(output_array_path, layer)
                cv.imwrite(output_image_path, apply_colormap(layer, cv.COLORMAP_VIRIDIS))
                layers_data.append(LayerData(output_image_path, output_array_path, layer_index=i))

            output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_cluster")
            output_image_path = f"{output_path_no_ext}.png"
            output_array_path = f"{output_path_no_ext}.npy"

            np.save(output_array_path, cluster)
            cv.imwrite(output_image_path, apply_colormap(cluster, cv.COLORMAP_JET))

            self.signals.progress.emit(index + 1,
                                       (output_image_path, output_array_path, input_basename_no_ext, layers_data))
        self.signals.finished.emit()

    @Slot()
    def interrupt(self) -> None:
        self.is_running = False


class MainWindow(PreviewWindow):
    """
    Extends PreviewWindow. The first window that is launched when the program starts.
    """

    def __init__(self):
        super(MainWindow, self).__init__(None)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.thread_pool = QThreadPool()

        self.input_directory = ""
        self.output_directory = ""
        self.merger_directory: Optional[str] = None
        self.editor_window: Optional[ClusterEditor] = None

        self.__generated_images_entries: List[ClusterImageEntry] = []

        self.ui.buttonInputDir.clicked.connect(self.load_input_directory)
        self.ui.buttonOutputDir.clicked.connect(self.load_output_directory)
        self.ui.buttonGenerate.clicked.connect(self.generate_handler)
        self.ui.buttonCheckUncheck.clicked.connect(self.select_deselect)
        self.ui.buttonClearGenerated.clicked.connect(self.clear_generated)

    def source_layout(self) -> QLayout:
        return self.ui.scrollAreaWidgetContentsSrc.layout()

    def image_preview(self) -> QLabel:
        return self.ui.imagePreview

    def clear_image_entries(self) -> None:
        super(MainWindow, self).clear_image_entries()
        for ime in self.__generated_images_entries:
            ime.close()
        self.__generated_images_entries.clear()

    @Slot(ClusterImageEntry)
    def open_cluster_editor(self, calling_image_entry: ClusterImageEntry) -> None:
        """
        Open a cluster editor with the given ClusteringImageEntry.

        :param ClusterImageEntry calling_image_entry: The cluster image entry which contains all layers information.
        """
        if self.editor_window is not None and self.editor_window.isVisible():
            self.editor_window.activateWindow()
            return
        self.editor_window = ClusterEditor(self, calling_image_entry)
        self.editor_window.applied_to_all.connect(self.merge_layers)
        self.editor_window.show()

    @Slot(list)
    def merge_layers(self, layers_indices: List[int]) -> None:
        """
        Merge all the specified layers

        :param layers_indices: A range of the layers to merge
        :type layers_indices: list of int
        """
        if len(layers_indices) == 0:
            return

        merged_cluster_ime: List[ClusterImageEntry] = []

        first = True
        while len(self.__generated_images_entries) > 0:
            ime: ClusterImageEntry = self.__generated_images_entries.pop(0)
            merged: Optional[np.ndarray] = None
            parent_layers: List[int] = []
            for i in layers_indices:
                layer_data: LayerData = ime.get_layer_data(i)
                if layer_data.is_merger:
                    assert layer_data.parent_layers is not None
                    parent_layers.extend(layer_data.parent_layers)
                else:
                    assert layer_data.layer_index is not None
                    parent_layers.append(layer_data.layer_index)

                layer = np.load(layer_data.array_path)
                merged = layer if merged is None else merged | layer

            for i in sorted(layers_indices, reverse=True):
                ime.remove_layer_data(i)

            if merged is None:
                break

            colored = apply_colormap(merged)

            merged_path_no_ext = os.path.join(self.merger_directory,
                                              f"{ime.basename}_layers_{LayerData.indices2str(parent_layers)}")
            merged_image_path = f"{merged_path_no_ext}.png"
            merged_array_path = f"{merged_path_no_ext}.npy"

            cv.imwrite(merged_image_path, colored)
            np.save(merged_array_path, merged)
            ime.add_layer_data(
                LayerData(merged_image_path, merged_array_path, is_merger=True, parent_layers=parent_layers))

            new_cluster_array = create_cluster(
                [np.load(ime.get_layer_data(i).array_path) for i in range(ime.layer_count())])

            new_cluster_colored = apply_colormap(new_cluster_array, cv.COLORMAP_JET)
            new_cluster_colored_image = array3d_to_pixmap(new_cluster_colored).toImage()

            new_cluster_path_no_ext = os.path.join(self.merger_directory, f"{ime.basename}_cluster")
            new_cluster_image_path = f"{new_cluster_path_no_ext}.png"
            new_cluster_array_path = f"{new_cluster_path_no_ext}.npy"

            np.save(new_cluster_array_path, new_cluster_array)
            cv.imwrite(new_cluster_image_path, new_cluster_colored)

            new_ime = ClusterImageEntry(ime.parent(), new_cluster_colored_image, ime.basename, new_cluster_image_path,
                                        new_cluster_array_path, ime.layers_data)
            new_ime.mouse_pressed.connect(self.image_entry_click_handler)
            new_ime.double_clicked.connect(self.open_cluster_editor)
            merged_cluster_ime.append(new_ime)
            ime.close()
            self.ui.scrollAreaWidgetContentsDst.layout().addWidget(new_ime)

            if first:
                self.set_preview_image(new_cluster_colored_image, new_ime)
                first = False

        self.__generated_images_entries = merged_cluster_ime

    # Need to be implemented if the unmerge feature is needed__mouse_pressed_handlers.clear()
    def unmerge_layer(self) -> None:
        raise NotImplementedError

    @Slot()
    def clear_generated(self) -> None:
        """
        Clear the generated cluster area.
        """
        for ime in self.__generated_images_entries:
            ime.close()
        self.__generated_images_entries.clear()
        self.image_preview().setText("Preview")
        self._selected_image_entry = None

    @Slot()
    def load_input_directory(self) -> None:
        """
        Open a popup to select the directory where is located source images.
        """
        in_dir = QFileDialog.getExistingDirectory(self)
        if len(in_dir) == 0:
            return

        self.input_directory = in_dir
        self.clear_image_entries()
        self.clear_preview_image()

        # self.ui.labelInDir.setText(f"Loaded: {self.input_directory}")

        src_layout = self.source_layout()
        first = True
        for entry in sorted(os.scandir(self.input_directory), key=lambda x: x.name):
            if not Tat.is_image(entry.path):
                continue

            qim = load_image(entry.path)

            ime = CheckableImageEntry(src_layout.parent(), qim, entry.name, entry.path)
            ime.mouse_pressed.connect(self.image_entry_click_handler)
            self.add_source_image_entry(ime)

            if first:
                self.set_preview_image(qim, ime)
                first = False

        if len(self.output_directory) != 0:
            self.ui.buttonGenerate.setEnabled(True)

    @Slot()
    def load_output_directory(self) -> None:
        """
        Open a popup to select the directory where to generate the clusters and layers.
        """
        out_dir = QFileDialog.getExistingDirectory(self)
        if len(out_dir) == 0:
            return

        self.output_directory = out_dir
        self.merger_directory = os.path.join(self.output_directory, "merged")
        Path(self.merger_directory).mkdir(exist_ok=True)

        # self.ui.labelOutDir.setText(f"Loaded: {self.output_directory}")
        if len(self.input_directory) != 0:
            self.ui.buttonGenerate.setEnabled(True)

    @Slot()
    def generate_handler(self) -> None:
        """
        When the generate button is clicked, starts the clustering in background and shows the progress bar.
        """
        progress_window = ProgressWindow()
        self.ui.buttonGenerate.setEnabled(False)
        self.setDisabled(True)

        @Slot(int, tuple)
        def add_cluster_image(progress: int, data: Tuple[str, str, str, List[LayerData]]) -> None:
            image_path, array_path, name, layers_data = data
            container: QLayout = self.ui.scrollAreaWidgetContentsDst.layout()
            ime = ClusterImageEntry(container.parent(), load_image(image_path), name, image_path, array_path,
                                    layers_data)
            ime.mouse_pressed.connect(self.image_entry_click_handler)
            ime.double_clicked.connect(self.open_cluster_editor)
            container.addWidget(ime)
            if len(self.__generated_images_entries) == 0:
                self.set_preview_image(load_image(ime.image_path), ime)
            progress_window.progress_bar().setValue(progress)
            self.__generated_images_entries.append(ime)

        selected_entries = self.get_selected_entries()
        worker = ClusteringWorker(selected_entries, self.ui.clusterCount.value(), self.ui.runCount.value(),
                                  self.ui.maxIterCount.value(), self.output_directory)

        @Slot()
        def finished_generating() -> None:
            progress_window.close()
            worker.signals.progress.disconnect(add_cluster_image)
            worker.signals.finished.disconnect(finished_generating)
            self.setEnabled(True)
            self.ui.buttonGenerate.setEnabled(True)

        worker.signals.progress.connect(add_cluster_image)
        worker.signals.finished.connect(finished_generating)
        progress_window.cancelled.connect(worker.interrupt)

        progress_window.progress_bar().setMaximum(len(selected_entries))
        progress_window.show()

        self.thread_pool.start(worker)



from PySide6.QtGui import QPixmap, QImage
from PySide6.QtCore import QSize

import numpy as np
import cv2 as cv

from typing import Optional, List


def fit_to_frame(pixmap: QPixmap, frame: QSize) -> QPixmap:
    """
    :param pixmap: The QPixmap to resize.
    :param frame: The frame within which the image should fit.
    :return: A pixmap fitting inside the given frame.
    """
    frame_width, frame_height = frame.toTuple()
    dw = abs(pixmap.width() - frame_width)
    dh = abs(pixmap.height() - frame_height)
    return pixmap.scaledToWidth(frame_width) if dw > dh else pixmap.scaledToHeight(frame_height)


def load_image(path: str) -> QImage:
    return QImage(path)


def apply_colormap(image: np.ndarray, colormap=cv.COLORMAP_VIRIDIS) -> np.ndarray:
    """
    Normalize array and apply colormap.

    :param numpy.ndarray image: The array to colorize.
    :param cv2.COLORMAP colormap: The colormap to apply.
    :return: Colored array with the given colormap.
    """
    image = image - np.min(image)
    return cv.applyColorMap(np.uint8(255 * (image / np.max(image))), colormap)


def array2d_to_pixmap(array: np.ndarray, normalize=False, colormap: int = cv.COLORMAP_VIRIDIS) -> QPixmap:
    """
    Convert a 2D array (monochrome image) to a QPixmap

    :param numpy.ndarray array: The array to convert.
    :param bool normalize: If `True` then apply colormap.
    :param colormap: Colomap used if normalize is `True`
    :return: QPixmap containing the array.
    """
    assert array.ndim == 2
    if normalize:
        array = apply_colormap(array, colormap)
        height, width, color_bytes = array.shape
        return QPixmap.fromImage(QImage(array.data, width, height, color_bytes * width, QImage.Format_BGR888))
    height, width = array.shape
    return QPixmap.fromImage(QImage(array.data, width, height, width, QImage.Format_Grayscale8))


def array3d_to_pixmap(array: np.ndarray) -> QPixmap:
    """
    Convert a 3D array (color image) to a QPixmap.

    :param array: The array to convert.
    :return: QPixmap containing the array.
    """
    assert array.ndim == 3
    height, width, color_bytes = array.shape
    return QPixmap.fromImage(QImage(array.data, width, height, color_bytes * width, QImage.Format_BGR888))


def create_cluster(layers: List[np.ndarray], normalized=False) -> Optional[np.ndarray]:
    """
    Create a cluster array from a list of layers.

    :param layers: The layers to cluster.
    :param normalized: If `True`, then normalize the resulting array.
    :return: A cluster with each layer on a centroid.
    """
    cluster: Optional[np.ndarray] = None
    for index, layer in enumerate(layers):
        cluster = layer.copy() if cluster is None else cluster + layer * (index + 1)
    return cluster / np.max(cluster) if normalized else cluster



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import sys
import os
import io
import re
import hashlib
from pbr.git import _iter_log_oneline, _iter_changelog, _run_git_command

sys.path.insert(0, os.path.abspath(os.path.join('..', 'src')))

# -- Project information -----------------------------------------------------

project = 'Tomography Analysis Tool'
copyright = '2021, Hugo Haldi'
author = 'Hugo Haldi'

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'myst_parser'
]

myst_enable_extensions = [
    'colon_fence'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_material'

html_theme_options = {
    'base_url': 'https://github.com/ShinoYasx/tat/',
    'repo_url': 'https://github.com/ShinoYasx/tat/',
    'repo_name': 'Tomography Analysis Tool',
    # 'google_analytics_account': 'UA-XXXXX',
    'html_minify': True,
    'css_minify': True,
    'nav_title': 'Tomography Analysis Tool',
    'logo_icon': '&#xe869',
    'globaltoc_depth': 2
}

html_sidebars = {
    "**": ["logo-text.html", "globaltoc.html", "localtoc.html", "searchbox.html"]
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


def sha256_file(file):
    sha256 = hashlib.sha256()
    with open(file, 'rb') as f:
        sha256.update(f.read())
    return sha256


def generate_changelog(*args, **kwargs):
    changelog = _iter_log_oneline(git_dir=os.path.join('..', '.git'))
    if changelog:
        changelog = _iter_changelog(changelog)
    if not changelog:
        return

    new_changelog = 'changelog.rst'

    new_content = bytearray()
    for _, content in changelog:
        new_content += content.encode('utf-8')

    if os.path.isfile(new_changelog) and hashlib.sha256(new_content).digest() == sha256_file(new_changelog).digest():
        return

    with io.open(new_changelog, "wb") as changelog_file:
        changelog_file.write(new_content)


def generate_authors(*args, **kwargs):
    old_authors = 'AUTHORS.in'
    new_authors = 'authors.md'
    if os.path.isfile(new_authors) and not os.access(new_authors, os.W_OK):
        # If there's already an AUTHORS file and it's not writable, just use it
        return

    ignore_emails = '((jenkins|zuul)@review|infra@lists|jenkins@openstack)'
    git_dir = os.path.join('..', '.git')
    authors = []

    # don't include jenkins email address in AUTHORS file
    git_log_cmd = ['log', '--format=%aN <%aE>']
    authors += _run_git_command(git_log_cmd, git_dir).split('\n')
    authors = [a for a in authors if not re.search(ignore_emails, a)]

    # get all co-authors from commit messages
    co_authors_out = _run_git_command('log', git_dir)
    co_authors = re.findall('Co-authored-by:.+', co_authors_out,
                            re.MULTILINE)
    co_authors = [signed.split(":", 1)[1].strip()
                  for signed in co_authors if signed]

    authors += co_authors
    authors = sorted(set(authors))

    new_authors_str = bytearray()

    if os.path.exists(old_authors):
        with open(old_authors, "rb") as old_authors_fh:
            new_authors_str += old_authors_fh.read()
    new_authors_str += (os.linesep.join(authors) + os.linesep).encode('utf-8')

    if os.path.isfile(new_authors) and hashlib.sha256(new_authors_str).digest() == sha256_file(new_authors).digest():
        return

    with open(new_authors, 'wb') as f:
        f.write(new_authors_str)


def on_builder_inited(*args, **kwargs):
    generate_changelog()
    # generate_authors()


def setup(app):
    app.connect('builder-inited', on_builder_inited)



from typing import Optional, Final, List

import numpy as np

from tat.checkable_image_entry import CheckableImageEntry
from tat.layer_data import LayerData

from PySide6.QtWidgets import QWidget
from PySide6.QtGui import QImage


class LayerImageEntry(CheckableImageEntry):
    """
    Extends CheckableImageEntry. Add a LayerData and an array-like to a CheckableImageEntry.
    """

    def __init__(self, parent: QWidget, image: QImage, array: np.ndarray, name: str, is_merger: bool = False,
                 layer_index: Optional[int] = None, parent_layers: Optional[List[int]] = None):
        """

        :param QWidget parent: The parent containing the ImageEntry.
        :param QImage image: The image that will be draw in the thumbnail.
        :param numpy.ndarray array: Array of the image.
        :param str name: The name that will be shown below the thumbnail. Also used for image basename.
        :param bool is_merger: Tells if the layer a merger of multiple layers or not.
        :param layer_index: Index of the layer if it is not a merger.
        :type layer_index: int, optional
        :param parent_layers: List of the parent layers if the layer is a merger.
        :type parent_layers: list of int, optional
        """
        super(LayerImageEntry, self).__init__(parent, image, name, default_check=False)

        self.array: Final[np.ndarray] = array
        self.layer_data: Final[LayerData] = LayerData(is_merger=is_merger, layer_index=layer_index,
                                                      parent_layers=parent_layers)



from typing import Optional, List
import sys
from PySide6.QtWidgets import QApplication
from tat.main_window import MainWindow


def main(args: Optional[List[str]] = None) -> int:
    if args is None:
        args = sys.argv[1:]
    # args is currently not used

    app = QApplication()
    app.main_window = MainWindow()
    app.main_window.show()
    return app.exec()



from typing import Optional, Final, List


class LayerData:
    """
    Container for layer data.
    """

    def __init__(self, image_path: Optional[str] = None, array_path: Optional[str] = None, is_merger=False,
                 layer_index: Optional[int] = None, parent_layers: Optional[List[int]] = None):
        """
        Creates a LayerData.

        :type image_path: str, optional
        :type array_path: str, optional
        :param bool is_merger: Tells if the layer a merger of multiple layers or not.
        :param layer_index: Index of the layer if it is not a merger.
        :type layer_index: int, optional
        :param parent_layers: List of the parent layers if the layer is a merger.
        :type parent_layers: list of int, optional
        """
        if is_merger:
            assert parent_layers is not None and layer_index is None
        else:
            assert parent_layers is None and layer_index is not None
        self.image_path: Final[str] = image_path
        self.array_path: Final[str] = array_path
        self.is_merger: Final[bool] = is_merger
        self.parent_layers: Final[Optional[List[int]]] = parent_layers
        self.layer_index: Final[Optional[int]] = layer_index

    def name(self) -> str:
        """
        :return: Name based on the index.
        :rtype: str
        """
        if self.is_merger:
            assert self.parent_layers is not None
            return "m " + self.indices2str(self.parent_layers)
        assert self.layer_index is not None
        return str(self.layer_index)

    @staticmethod
    def indices2str(indices: List[int]) -> str:
        """
        Creates a string from a list of indices, each index separated by a `+`.

        :type indices: list of int
        :rtype: str
        """
        indices_str = ""
        for i in indices:
            indices_str += str(i) if i == indices[0] else f"+{str(i)}"
        return indices_str



import sys
import os

# If we are not running as a package, add project root to path
if not __package__:
    path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, path)

if __name__ == "__main__":
    from tat.gui.main import main as _main
    sys.exit(_main())



from __future__ import annotations
from typing import Callable, Final, Any, Optional, List
from tat.utils import fit_to_frame
from textwrap import wrap

from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel, QSizePolicy
from PySide6.QtGui import QImage, QPixmap, QColor, QMouseEvent
from PySide6.QtCore import Qt, QSize, Signal


class ImageEntry(QWidget):
    """
    Extends QWidget. ImageEntry is a widget that contains a thumbnail and store image information.
    """

    mouse_pressed = Signal(QWidget, QMouseEvent)

    def __init__(self, parent: QWidget, image: QImage, name: str, image_path: Optional[str] = None,
                 array_path: Optional[str] = None):
        """
        Create an ImageEntry

        :param QWidget parent: The parent containing the ImageEntry.
        :param QImage image: The image that will be draw in the thumbnail.
        :param str name: The name that will be shown below the thumbnail. Also used for image basename.
        :param image_path: Path to the image file.
        :type image_path: str, optional
        :param array_path: Path to the Numpy array file.
        :type array_path: str, optional
        """
        super(ImageEntry, self).__init__(parent)
        self.__selected = False
        self.__mouse_pressed_handlers: List[Callable[[ImageEntry, QMouseEvent], Any]] = []
        self.setAutoFillBackground(True)
        self.__default_background_color = self.palette().color(self.backgroundRole())
        self.image_path: Final[Optional[str]] = image_path
        self.array_path: Final[Optional[str]] = array_path
        self.basename: Final[str] = name

        thumbnail = QLabel("Thumbnail", self)
        thumbnail.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        thumbnail.setAlignment(Qt.AlignCenter)
        thumbnail.setPixmap(fit_to_frame(QPixmap.fromImage(image), QSize(50, 50)))

        name_label = QLabel('\n'.join(wrap(name, 15)), self)
        name_label.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum)
        name_label.setAlignment(Qt.AlignCenter)

        layout = QVBoxLayout(self)
        layout.addWidget(thumbnail, alignment=Qt.AlignHCenter)
        layout.addWidget(name_label, alignment=Qt.AlignHCenter)
        self.setLayout(layout)

    def setSelected(self, selected: bool) -> None:
        """
        Set the ImageEntry background to gray if selected is True, else set to the original color.

        :param bool selected: Path to the image file.
        """
        if selected:
            self.__setBackgroundColor(Qt.gray)
            self.__selected = True
            return

        self.__setBackgroundColor(self.__default_background_color)
        self.__selected = False

    def isSelected(self) -> bool:
        """
        :return: True if ImageEntry is currently selected.
        :rtype: bool
        """
        return self.__selected

    def __setBackgroundColor(self, color: QColor) -> None:
        """
        Change the background color to the given parameter.

        :param QColor color:
        """
        pal = self.palette()
        pal.setColor(self.backgroundRole(), color)
        self.setPalette(pal)

    def mousePressEvent(self, event: QMouseEvent) -> None:
        self.mouse_pressed.emit(self, event)



from PySide6.QtWidgets import QWidget, QCheckBox, QSizePolicy
from PySide6.QtGui import QImage
from PySide6.QtCore import Qt, Signal

from tat.image_entry import ImageEntry
from typing import Optional


class CheckableImageEntry(ImageEntry):
    """
    Extends ImageEntry. An ImageEntry with a checkbox under the thumbnail.
    """

    state_changed = Signal(int)

    def __init__(self, parent: QWidget, image: QImage, name: str, image_path: Optional[str] = None,
                 array_path: Optional[str] = None, default_check: bool = True):
        """
        Initialize a CheckableImageEntry

        :param QWidget parent: The parent containing the ImageEntry.
        :param QImage image: The image that will be draw in the thumbnail.
        :param str name: The name that will be shown below the thumbnail. Also used for image basename.
        :param image_path: Path to the image file.
        :type image_path: str, optional
        :param array_path: Path to the Numpy array file.
        :type array_path: str, optional
        :param bool default_check: If True, the checkbox will be checked.
        """
        super(CheckableImageEntry, self).__init__(parent, image, name, image_path, array_path)

        self.__check_box = QCheckBox(self)
        self.__check_box.setSizePolicy(
            QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.__check_box.setChecked(default_check)
        self.__check_box.stateChanged.connect(self.state_changed)

        self.layout().insertWidget(1, self.__check_box, alignment=Qt.AlignHCenter)

    def isChecked(self) -> bool:
        """
        :return: `True` if checkbox is checked, `False` otherwise.
        :rtype: bool
        """
        return self.__check_box.isChecked()

    def setChecked(self, checked) -> None:
        """
        Changes the entry checkbox state
        :param checked: If `True`, set the checkbox to checked, otherwise set to unchecked.
        """
        self.__check_box.setChecked(checked)



import sys
from cx_Freeze import setup, Executable

# base="Win32GUI" should be used only for Windows GUI app
base = None
if sys.platform == "win32":
    base = "Win32GUI"

version = ""
with open("version.txt") as f:
    version = f.read().rstrip("\n")

sys.path.insert(0, "src")

shortcut_table = [
    ("DesktopShortcut",             # Shortcut
     "DesktopFolder",               # Directory_
     "Tomography Analysis Tool",    # Name
     "TARGETDIR",                   # Component_
     "[TARGETDIR]tat.exe",          # Target
     None,                          # Arguments
     None,                          # Description
     None,                          # Hotkey
     None,                          # Icon
     None,                          # IconIndex
     None,                          # ShowCmd
     'TARGETDIR'                    # WkDir
     )
]

msi_data = {
    "Shortcut": shortcut_table
}

bdist_msi_options = {
    "summary_data": {
        "author": "Hugo Haldi",
        "comments": "Tomography Analysis Tool",
        "keywords": "TAT Tomography Analysis Tool"
    },
    "data": msi_data,
    "upgrade_code": "{22872b66-f52b-40fd-b973-951a9cac3620}"
}

build_exe_options = {
    "packages": [
        "PySide6",
        "cv2",
        "skimage",
        "tat"
    ],
    "excludes": [
        "tkinter"
    ],
    "path": sys.path
}

executables = [
    Executable("src/tat/__main__.py",
               copyright="Copyright (C) 2022 TAT",
               base=base,
               target_name="tat",
               shortcut_name="Tomography Analysis Tool")
]

setup(
    name="tat",
    version=version,
    description="Tomography Analysis Tool",
    options={
        "build_exe": build_exe_options,
        "bdist_msi": bdist_msi_options
    },
    executables=executables
)



from typing import Optional, Union, List, Tuple

import os

import cv2 as cv
import numpy as np

from PySide6.QtWidgets import QLayout, QLabel, QWidget, QGridLayout
from PySide6.QtGui import QImage, QMouseEvent, QCloseEvent, QResizeEvent, QMoveEvent, QPixmap
from PySide6.QtCore import Slot, QSize, QPoint, Qt, Signal

from tat.ui_editor_window import Ui_EditorWindow
from tat.preview_window import PreviewWindow
from tat.cluster_image_entry import ClusterImageEntry
from tat.layer_image_entry import LayerImageEntry
from tat.layer_data import LayerData
from tat.utils import load_image, array2d_to_pixmap, fit_to_frame, create_cluster


class CLusterPreviewWindow(QWidget):
    """
    Extends QWidget. Floating window next to ClusterEditor showing the current cluster state.
    """

    def __init__(self, parent: Optional[QWidget] = None, size: QSize = QSize(600, 600), image: Optional[QImage] = None):
        super(CLusterPreviewWindow, self).__init__(parent)
        self.setWindowFlags(Qt.Window | Qt.FramelessWindowHint)
        self.resize(size)

        self.imageLabel = QLabel("Cluster Preview", self)
        self.imageLabel.setAlignment(Qt.AlignCenter)

        layout = QGridLayout(self)
        layout.addWidget(self.imageLabel)

        if image is not None:
            self.__update_cluster_preview(QPixmap.fromImage(image))

    def __update_cluster_preview(self, image: QPixmap) -> None:
        self.imageLabel.setPixmap(fit_to_frame(image, QSize(self.width(), self.height())))

    def update_cluster_preview(self, image: Union[np.ndarray, str]) -> None:
        """
        Load an image from a string or an array and update the cluster preview.

        :param image: Can be both a numpy array and a string.
        """
        if isinstance(image, np.ndarray):
            self.__update_cluster_preview(array2d_to_pixmap(image, normalize=True, colormap=cv.COLORMAP_JET))
            return

        if isinstance(image, str):
            self.__update_cluster_preview(QPixmap.fromImage(QImage(image)))
            return

        raise ValueError("Invalid image type: {}".format(type(image)))


class ClusterEditor(PreviewWindow):
    """
    Extends PreviewWindow. The window that allows editing of the clusters.
    """

    applied_to_all = Signal(list)

    def __init__(self, parent: Optional[QWidget], calling_image_entry: ClusterImageEntry):
        super(ClusterEditor, self).__init__(parent)
        self.ui = Ui_EditorWindow()
        self.ui.setupUi(self)
        self.ui.mergeButton.clicked.connect(self.merge)
        self.ui.applyButton.clicked.connect(self.apply_to_all)
        self.ui.resetButton.clicked.connect(self.reset)
        # self.ui.unmergeButton.clicked.connect(self.unmerge)
        self.ui.undoButton.clicked.connect(self.undo)

        self._source_image_entries: List[LayerImageEntry] = []
        self._selected_image_entry: Optional[LayerImageEntry] = None
        self.__cluster_image_entry: ClusterImageEntry = calling_image_entry
        self.__pending_mergers: List[List[int]] = []
        self.__pending_ime: List[LayerImageEntry] = []
        self.__old_entries: List[List[LayerImageEntry]] = []
        self.__cluster_array: np.ndarray = np.load(self.__cluster_image_entry.array_path)

        side_length = self.height() - self.menuBar().height()
        self.__cluster_preview_window = CLusterPreviewWindow(self, QSize(side_length, side_length),
                                                             load_image(self.__cluster_image_entry.image_path))
        # self.cluster_preview_window.show()

        # first = True
        for i in range(self.__cluster_image_entry.layer_count()):
            layer_data = self.__cluster_image_entry.get_layer_data(i)
            array = np.load(layer_data.array_path)
            qim: QImage = load_image(layer_data.image_path)
            ime = LayerImageEntry(self, qim, array, layer_data.name(), is_merger=layer_data.is_merger,
                                  layer_index=layer_data.layer_index, parent_layers=layer_data.parent_layers)
            ime.mouse_pressed.connect(self.image_entry_click_handler)
            ime.state_changed.connect(self.change_merge_button_state)
            self.add_source_image_entry(ime)

            # if first:
            # self.set_preview_image(qim, ime)
            # first = False

    def source_layout(self) -> QLayout:
        return self.ui.scrollAreaLayersContents.layout()

    def image_preview(self) -> QLabel:
        return self.ui.imageLabel

    @Slot(LayerImageEntry, QMouseEvent)
    def image_entry_click_handler(self, sender: LayerImageEntry, event: QMouseEvent) -> None:
        assert type(sender) == LayerImageEntry
        self.set_preview_image(array2d_to_pixmap(sender.array, normalize=True).toImage(), sender)

    def resizeEvent(self, event: QResizeEvent) -> None:
        if self._selected_image_entry is None:
            return
        self.draw_preview_image(array2d_to_pixmap(self._selected_image_entry.array, normalize=True).toImage())

    def moveEvent(self, event: QMoveEvent) -> None:
        position = event.pos()
        self.__cluster_preview_window.move(position - QPoint(self.__cluster_preview_window.width(), 0))
        if self.__cluster_preview_window.isHidden():
            self.__cluster_preview_window.show()

    def closeEvent(self, event: QCloseEvent) -> None:
        self.__cluster_preview_window.close()
        self.__cluster_preview_window = None

    def __pending_add(self, mergers_idx: List[int], ime: LayerImageEntry, old_entries: List[LayerImageEntry]) -> None:
        """
        Add the result of a merge to the pending list, and store the merged layers to be able to undo the merge.

        :param mergers_idx: Indices of the layers to merge.
        :type mergers_idx: list of int
        :param LayerImageEntry ime: The newly merged image entry.
        :param old_entries: A list of the layers used to generate the merged layer.
        :type old_entries: list of LayerImageEntry
        """
        if not self.ui.undoButton.isEnabled():
            self.ui.undoButton.setEnabled(True)
        self.__pending_mergers.append(mergers_idx)
        self.__pending_ime.append(ime)
        self.__old_entries.append(old_entries)

    def __pending_clear(self) -> None:
        """
        Deletes the mergers that haven't been applied yet.
        """
        self.ui.undoButton.setEnabled(False)
        self.__pending_mergers.clear()
        self.__pending_ime.clear()
        self.__old_entries.clear()

    def __pending_count(self) -> int:
        n = len(self.__pending_mergers)
        assert n == len(self.__pending_ime) == len(self.__old_entries)
        return n

    def __pending_pop(self) -> Tuple[List[int], LayerImageEntry, List[LayerImageEntry]]:
        """
        Gives a tuple of the last merged indices, image entry of the merged layers and the list of image entries
        used to generate the merger.

        :rtype: (list of int, LayerImageEntry, list of LayerImageEntry)
        """
        if self.__pending_count() == 1:
            self.ui.undoButton.setEnabled(False)
        return self.__pending_mergers.pop(), self.__pending_ime.pop(), self.__old_entries.pop()

    @Slot(int)
    def change_merge_button_state(self, state: int) -> None:
        if not state == Qt.CheckState.Checked:
            self.ui.mergeButton.setEnabled(True)
            return
        if len(self.get_selected_entries()) == len(self._source_image_entries):
            self.ui.mergeButton.setEnabled(False)

    @Slot()
    def merge(self) -> None:
        """
        Merge the selected layers only in the current view. Update the cluster preview with the newly merged cluster.
        """
        if len(self.get_selected_entries()) < 2:
            return
        checked_indices: List[int] = []
        old_ime: List[LayerImageEntry] = []
        merger: Optional[np.ndarray] = None
        parent_layers: List[int] = []
        for index, ime in enumerate(self._source_image_entries):
            if not ime.isChecked():
                continue

            if ime.layer_data.is_merger:
                assert ime.layer_data.parent_layers is not None
                parent_layers.extend(ime.layer_data.parent_layers)
            else:
                assert ime.layer_data.layer_index is not None
                parent_layers.append(ime.layer_data.layer_index)

            checked_indices.append(index)
            old_ime.append(ime)
            merger = self._source_image_entries[index].array if merger is None else merger | self._source_image_entries[
                index].array
            ime.setChecked(False)
            ime.close()

        for i in sorted(checked_indices, reverse=True):
            self._source_image_entries.pop(i)

        self.__cluster_array = create_cluster([ime.array for ime in self._source_image_entries])
        self.__cluster_preview_window.update_cluster_preview(self.__cluster_array)

        qim: QImage = array2d_to_pixmap(merger, normalize=True).toImage()
        merged_ime = LayerImageEntry(self, qim, merger, f"m {LayerData.indices2str(parent_layers)}",
                                     is_merger=True, parent_layers=parent_layers)
        merged_ime.mouse_pressed.connect(self.image_entry_click_handler)
        merged_ime.state_changed.connect(self.change_merge_button_state)
        self.__pending_add(checked_indices, merged_ime, old_ime)
        self.set_preview_image(qim, merged_ime)
        self.add_source_image_entry(merged_ime)
        self.change_all_entries_check_state(False)

    @Slot()
    def apply_to_all(self) -> None:
        """
        Send a merge signal for each pending merge.
        """
        for merger in self.__pending_mergers:
            self.applied_to_all.emit(merger)
        self.__pending_clear()

    @Slot()
    def reset(self) -> None:
        """
        Removes all uncommitted changes done in the editor.
        """
        if len(self.__pending_mergers) == 0:
            return

        self.__pending_clear()

        for ime in self._source_image_entries:
            ime.close()

        self._source_image_entries.clear()

        self.__cluster_array = np.load(self.__cluster_image_entry.array_path)
        self.__cluster_preview_window.update_cluster_preview(self.__cluster_array)

        for i in range(self.__cluster_image_entry.layer_count()):
            layer_data = self.__cluster_image_entry.get_layer_data(i)
            array = np.load(layer_data.array_path)
            qim: QImage = load_image(layer_data.image_path)
            ime = LayerImageEntry(self, qim, array, layer_data.name(), layer_data.is_merger, layer_data.layer_index,
                                  layer_data.parent_layers)
            ime.mouse_pressed.connect(self.image_entry_click_handler)
            ime.state_changed.connect(self.change_merge_button_state)
            self.add_source_image_entry(ime)
            if i == 0:
                self.set_preview_image(load_image(layer_data.image_path), ime)
        self.change_all_entries_check_state(False)

    @Slot()
    def unmerge(self) -> None:
        """
        Unmerge the selected layers in the editor. Global behavior not implemented.
        """
        for index, ime in enumerate(self._source_image_entries):
            if not ime.isChecked() or not ime.layer_data.is_merger:
                continue

            self._source_image_entries.pop(index)
            assert ime.layer_data.parent_layers is not None
            for parent_layer_index in ime.layer_data.parent_layers.copy():
                directory = os.path.dirname(self.__cluster_image_entry.image_path)
                path_no_ext = os.path.join(directory,
                                           f"{self.__cluster_image_entry.basename}_layer_{parent_layer_index}")
                image_path = f"{path_no_ext}.png"
                array_path = f"{path_no_ext}.npy"
                parent_ime = LayerImageEntry(self, load_image(image_path), np.load(array_path), str(parent_layer_index),
                                             layer_index=parent_layer_index)
                parent_ime.mouse_pressed.connect(self.image_entry_click_handler)
                parent_ime.state_changed.connect(self.change_merge_button_state)
                self.add_source_image_entry(parent_ime)
            ime.close()

    @Slot()
    def undo(self) -> None:
        """
        Go one step back.
        """
        if self.__pending_count() == 0:
            return

        indices, pending_ime, old_ime = self.__pending_pop()
        self._source_image_entries.pop()

        pending_ime.close()

        for index, ime in zip(indices, old_ime):
            ime.setVisible(True)
            self.add_source_image_entry(ime, index)
        self.image_preview().setText("Layer")
        self.__cluster_preview_window.update_cluster_preview(self.__cluster_image_entry.image_path)
        self.change_all_entries_check_state(False)



import numpy as np
from skimage import filters
from sklearn.cluster import KMeans
import time
import mimetypes
from typing import Optional, Tuple, List


class Tat:
    """
    Contains main methods for k-mean clustering.
    """

    @staticmethod
    def guess_type(filename: str) -> Optional[str]:
        filetype = mimetypes.guess_type(filename)[0]
        if filetype is None:
            return None
        return filetype.split("/")[0]

    @staticmethod
    def is_image(filename: str) -> bool:
        return Tat.guess_type(filename) == "image"

    @staticmethod
    def generate_layers(src_image: np.ndarray, cluster_count: int, run_count: int,
                        max_iter_count: int) -> Tuple[List[np.ndarray], np.ndarray]:
        """
        Computes the k-means for the given image and parameters.

        :param numpy.ndarray src_image: Input image to computes.
        :param int cluster_count: The number of clusters to form as well as the number of centroids to generate.
        :param run_count: Number of iterations run.
        :param max_iter_count: Maximum number of iterations of the k-means algorithm for a single run.
        :return: Tuple of a list of the layers and the clustered layers.
        :rtype: (list of numpy.ndarray, numpy.ndarray)
        """
        img_shape = src_image.shape
        start_time = time.time()
        k_means = KMeans(n_clusters=cluster_count, random_state=0, n_init=run_count, max_iter=max_iter_count).fit(
            filters.gaussian(src_image.reshape(img_shape[0] * img_shape[1], 1), 2))
        delta = time.time() - start_time
        print(f"k-means computing time: {delta}")
        segments = k_means.labels_.reshape(img_shape)
        labeled_segments = k_means.cluster_centers_[segments][:, :, 0]
        labeled_cluster = np.asarray(k_means.cluster_centers_)

        powers = np.floor(np.log10(labeled_segments))
        monochrome_labeled_segments = 100 ** powers * \
                                      np.floor(labeled_segments / 100 ** powers)

        powers_cluster = np.floor(np.log10(labeled_cluster))
        monochrome_labeled_cluster = np.sort(100 ** powers_cluster * np.floor(labeled_cluster / 100 ** powers_cluster),
                                             axis=0)

        layers: List[np.ndarray] = []
        for i in range(len(monochrome_labeled_cluster)):
            layer = np.zeros(shape=np.shape(
                monochrome_labeled_segments)).astype(np.uint8)
            layer[monochrome_labeled_segments ==
                  monochrome_labeled_cluster[i]] = 1
            layers.append(layer)
        return layers, monochrome_labeled_segments



#!/bin/env python

import os
import sys
from typing import List, Tuple


class RstGenerator:
    def __init__(self, package_name) -> None:
        self.package_name = package_name

    def generate_class_rst(self, module_name: str, class_name: str) -> str:
        s = class_name + os.linesep + len(class_name) * '=' + 2 * os.linesep + \
            f'.. autoclass:: {self.package_name}.{module_name}.{class_name}' + \
            os.linesep + '    :members:' + os.linesep + \
            '    :special-members: __init__' + os.linesep
        return s


if __name__ == "__main__":
    modules_classes: List[Tuple[str, str]] = [
        ('checkable_image_entry', 'CheckableImageEntry'),
        ('cluster_editor', 'ClusterEditor'),
        ('cluster_image_entry', 'ClusterImageEntry'),
        ('clustering', 'Tat'),
        ('image_entry', 'ImageEntry'),
        ('layer_data', 'LayerData'),
        ('layer_image_entry', 'LayerImageEntry'),
        ('main_window', 'MainWindow'),
        ('preview_window', 'PreviewWindow'),
        ('progress_window', 'ProgressWindow')
    ]

    generator = RstGenerator('tat')
    for module_name, class_name in modules_classes:
        with open(os.path.join('source', 'dev-guide', 'classes', f'{module_name}.rst'), 'wb') as file:
            file.write(generator.generate_class_rst(
                module_name, class_name).encode('utf-8'))



from PySide6.QtWidgets import QDialog, QWidget, QProgressBar
from PySide6.QtGui import QCloseEvent
from PySide6.QtCore import Signal

from typing import Optional

from tat.ui_progress_bar import Ui_ProgressBar


class ProgressWindow(QDialog):
    """
    Extends QDialog. Create a window with a progress bar and a cancel button.
    """

    cancelled = Signal()

    def __init__(self, parent: Optional[QWidget] = None):
        super().__init__(parent)
        self.__ui = Ui_ProgressBar()
        self.__ui.setupUi(self)
        self.__ui.cancelButton.clicked.connect(self.cancelled)

    def closeEvent(self, event: QCloseEvent) -> None:
        self.cancelled.emit()

    def progress_bar(self) -> QProgressBar:
        return self.__ui.progressBar



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_window.ui'
##
## Created by: Qt User Interface Compiler version 6.1.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(800, 700)
        sizePolicy = QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy1)
        self.centralwidget.setMinimumSize(QSize(0, 554))
        self.centralwidget.setLayoutDirection(Qt.LeftToRight)
        self.centralwidget.setAutoFillBackground(False)
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(9, -1, -1, -1)
        self.formLayout = QFormLayout()
        self.formLayout.setObjectName(u"formLayout")
        self.formLayout.setVerticalSpacing(6)
        self.labelClusterCount = QLabel(self.centralwidget)
        self.labelClusterCount.setObjectName(u"labelClusterCount")
        sizePolicy1.setHeightForWidth(self.labelClusterCount.sizePolicy().hasHeightForWidth())
        self.labelClusterCount.setSizePolicy(sizePolicy1)

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.labelClusterCount)

        self.clusterCount = QSpinBox(self.centralwidget)
        self.clusterCount.setObjectName(u"clusterCount")
        sizePolicy1.setHeightForWidth(self.clusterCount.sizePolicy().hasHeightForWidth())
        self.clusterCount.setSizePolicy(sizePolicy1)
        self.clusterCount.setMaximum(1000)
        self.clusterCount.setValue(4)

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.clusterCount)

        self.labelRunCount = QLabel(self.centralwidget)
        self.labelRunCount.setObjectName(u"labelRunCount")
        sizePolicy1.setHeightForWidth(self.labelRunCount.sizePolicy().hasHeightForWidth())
        self.labelRunCount.setSizePolicy(sizePolicy1)

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.labelRunCount)

        self.runCount = QSpinBox(self.centralwidget)
        self.runCount.setObjectName(u"runCount")
        sizePolicy1.setHeightForWidth(self.runCount.sizePolicy().hasHeightForWidth())
        self.runCount.setSizePolicy(sizePolicy1)
        self.runCount.setMaximum(1000)
        self.runCount.setValue(5)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.runCount)

        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")
        sizePolicy1.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy1)

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.label)

        self.maxIterCount = QSpinBox(self.centralwidget)
        self.maxIterCount.setObjectName(u"maxIterCount")
        sizePolicy1.setHeightForWidth(self.maxIterCount.sizePolicy().hasHeightForWidth())
        self.maxIterCount.setSizePolicy(sizePolicy1)
        self.maxIterCount.setMaximum(1000)
        self.maxIterCount.setValue(100)

        self.formLayout.setWidget(2, QFormLayout.FieldRole, self.maxIterCount)

        self.buttonGenerate = QPushButton(self.centralwidget)
        self.buttonGenerate.setObjectName(u"buttonGenerate")
        self.buttonGenerate.setEnabled(False)
        sizePolicy1.setHeightForWidth(self.buttonGenerate.sizePolicy().hasHeightForWidth())
        self.buttonGenerate.setSizePolicy(sizePolicy1)
        self.buttonGenerate.setCheckable(False)
        self.buttonGenerate.setChecked(False)

        self.formLayout.setWidget(3, QFormLayout.SpanningRole, self.buttonGenerate)


        self.gridLayout.addLayout(self.formLayout, 19, 6, 1, 1)

        self.buttonClearGenerated = QPushButton(self.centralwidget)
        self.buttonClearGenerated.setObjectName(u"buttonClearGenerated")
        sizePolicy2 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.buttonClearGenerated.sizePolicy().hasHeightForWidth())
        self.buttonClearGenerated.setSizePolicy(sizePolicy2)
        self.buttonClearGenerated.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.buttonClearGenerated, 14, 6, 1, 1)

        self.scrollAreaSrc = QScrollArea(self.centralwidget)
        self.scrollAreaSrc.setObjectName(u"scrollAreaSrc")
        sizePolicy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.scrollAreaSrc.sizePolicy().hasHeightForWidth())
        self.scrollAreaSrc.setSizePolicy(sizePolicy3)
        self.scrollAreaSrc.setMinimumSize(QSize(150, 0))
        self.scrollAreaSrc.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollAreaSrc.setWidgetResizable(True)
        self.scrollAreaWidgetContentsSrc = QWidget()
        self.scrollAreaWidgetContentsSrc.setObjectName(u"scrollAreaWidgetContentsSrc")
        self.scrollAreaWidgetContentsSrc.setGeometry(QRect(0, 0, 622, 142))
        self.horizontalLayout = QHBoxLayout(self.scrollAreaWidgetContentsSrc)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.scrollAreaSrc.setWidget(self.scrollAreaWidgetContentsSrc)

        self.gridLayout.addWidget(self.scrollAreaSrc, 18, 0, 2, 1)

        self.buttonOutputDir = QPushButton(self.centralwidget)
        self.buttonOutputDir.setObjectName(u"buttonOutputDir")
        sizePolicy2.setHeightForWidth(self.buttonOutputDir.sizePolicy().hasHeightForWidth())
        self.buttonOutputDir.setSizePolicy(sizePolicy2)

        self.gridLayout.addWidget(self.buttonOutputDir, 18, 6, 1, 1)

        self.buttonInputDir = QPushButton(self.centralwidget)
        self.buttonInputDir.setObjectName(u"buttonInputDir")
        sizePolicy2.setHeightForWidth(self.buttonInputDir.sizePolicy().hasHeightForWidth())
        self.buttonInputDir.setSizePolicy(sizePolicy2)

        self.gridLayout.addWidget(self.buttonInputDir, 17, 6, 1, 1)

        self.buttonCheckUncheck = QPushButton(self.centralwidget)
        self.buttonCheckUncheck.setObjectName(u"buttonCheckUncheck")
        sizePolicy4 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
        sizePolicy4.setHorizontalStretch(0)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.buttonCheckUncheck.sizePolicy().hasHeightForWidth())
        self.buttonCheckUncheck.setSizePolicy(sizePolicy4)
        self.buttonCheckUncheck.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.buttonCheckUncheck, 17, 0, 1, 1)

        self.imageFrame = QFrame(self.centralwidget)
        self.imageFrame.setObjectName(u"imageFrame")
        sizePolicy5 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy5.setHorizontalStretch(0)
        sizePolicy5.setVerticalStretch(0)
        sizePolicy5.setHeightForWidth(self.imageFrame.sizePolicy().hasHeightForWidth())
        self.imageFrame.setSizePolicy(sizePolicy5)
        self.imageFrame.setFrameShape(QFrame.StyledPanel)
        self.imageFrame.setFrameShadow(QFrame.Raised)
        self.gridLayout_2 = QGridLayout(self.imageFrame)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.imagePreview = QLabel(self.imageFrame)
        self.imagePreview.setObjectName(u"imagePreview")
        sizePolicy1.setHeightForWidth(self.imagePreview.sizePolicy().hasHeightForWidth())
        self.imagePreview.setSizePolicy(sizePolicy1)
        self.imagePreview.setMinimumSize(QSize(300, 300))
        font = QFont()
        font.setPointSize(22)
        self.imagePreview.setFont(font)
        self.imagePreview.setAlignment(Qt.AlignCenter)

        self.gridLayout_2.addWidget(self.imagePreview, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.imageFrame, 2, 0, 13, 1)

        self.scrollAreaDst = QScrollArea(self.centralwidget)
        self.scrollAreaDst.setObjectName(u"scrollAreaDst")
        sizePolicy6 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)
        sizePolicy6.setHorizontalStretch(0)
        sizePolicy6.setVerticalStretch(0)
        sizePolicy6.setHeightForWidth(self.scrollAreaDst.sizePolicy().hasHeightForWidth())
        self.scrollAreaDst.setSizePolicy(sizePolicy6)
        self.scrollAreaDst.setMinimumSize(QSize(150, 0))
        self.scrollAreaDst.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollAreaDst.setWidgetResizable(True)
        self.scrollAreaWidgetContentsDst = QWidget()
        self.scrollAreaWidgetContentsDst.setObjectName(u"scrollAreaWidgetContentsDst")
        self.scrollAreaWidgetContentsDst.setGeometry(QRect(0, 0, 150, 472))
        self.verticalLayout = QVBoxLayout(self.scrollAreaWidgetContentsDst)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.scrollAreaDst.setWidget(self.scrollAreaWidgetContentsDst)

        self.gridLayout.addWidget(self.scrollAreaDst, 2, 6, 12, 1)

        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"TAT - Main window", None))
        self.labelClusterCount.setText(QCoreApplication.translate("MainWindow", u"Cluster Count", None))
        self.clusterCount.setSuffix("")
        self.clusterCount.setPrefix("")
        self.labelRunCount.setText(QCoreApplication.translate("MainWindow", u"Run count", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Max iterations", None))
        self.buttonGenerate.setText(QCoreApplication.translate("MainWindow", u"Generate", None))
        self.buttonClearGenerated.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.buttonOutputDir.setText(QCoreApplication.translate("MainWindow", u"Load output directory", None))
        self.buttonInputDir.setText(QCoreApplication.translate("MainWindow", u"Load input directory", None))
        self.buttonCheckUncheck.setText(QCoreApplication.translate("MainWindow", u"Select/Deselect all", None))
        self.imagePreview.setText(QCoreApplication.translate("MainWindow", u"Preview", None))
    # retranslateUi

